import pika
import os

RABBITMQ_DEFAULT_HOST = os.environ.get('RABBITMQ_DEFAULT_HOST')
RABBITMQ_DEFAULT_PORT = os.environ.get('RABBITMQ_DEFAULT_PORT')

RABBITMQ_DEFAULT_USER = os.environ.get('RABBITMQ_DEFAULT_USER')
RABBITMQ_DEFAULT_PASS = os.environ.get('RABBITMQ_DEFAULT_PASS')

def create_connection(**kwargs):
  host = kwargs['host'] if 'host' in kwargs else RABBITMQ_DEFAULT_HOST
  port = kwargs['port'] if 'port' in kwargs else RABBITMQ_DEFAULT_PORT
  user = kwargs['username'] if 'username' in kwargs else RABBITMQ_DEFAULT_USER
  passwd = kwargs['password'] if 'password' in kwargs else RABBITMQ_DEFAULT_PASS

  credentials = pika.PlainCredentials(user, passwd)
  connection = pika.BlockingConnection(pika.ConnectionParameters(host, port, '/', credentials))
  return connection.channel(), connection 



from bookshelf_common.events.objects.book_exchange import BookExchange
from bookshelf_common.events.objects.book_status_exchange import BookStatusExchange
from bookshelf_common.events.objects.user_exchange import UserExchange

from bookshelf_common.events.base_connection import create_connection
from bookshelf_common.events.base_listener import init_listener
from bookshelf_common.events.base_publisher import publish_message


from dataclasses import dataclass, asdict
import json

@dataclass
class UserExchange:
  id: int
  email: str
  first_name: str
  last_name: str
  address: str
  city: str
  phone: str
  action: str = 'create'


from bookshelf_common.events import create_connection

def init_listener(exchange_subject, queue_subject, callback, exchange_type='fanout', channel=None, **kwargs):
  if channel is None:
    channel, connection = create_connection(**kwargs)
  # Create fanout exchange
  channel.exchange_declare(exchange=exchange_subject, exchange_type=exchange_type)
  
  # Create queues and bind them to the exchange
  queue = channel.queue_declare(queue=queue_subject)
  channel.queue_bind(exchange=exchange_subject, queue=queue.method.queue)

  # Set up consumers
  channel.basic_consume(queue=queue.method.queue, on_message_callback=callback)
  
  print(' [*] Waiting for messages. To exit press CTRL+C')
  
  # Start consuming
  channel.start_consuming()

  
